#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/pci.h>


#define for_each_pci_dev_in_class(_class, mask, dev)  \
while ((dev = pci_get_device (PCI_ANY_ID, PCI_ANY_ID, dev)) != NULL) \
	if ((dev->class ^ _class) & mask)  	\
		continue;			\
	else


// ^ pci_get_class won't do cause we need a mask 

#define SERIAL_DRIVER	"serial"

static void detach_from_serial_class (unsigned int class, unsigned int mask, struct pci_device_id *ids)
{
	struct pci_dev *dev = NULL;
	const struct pci_device_id *entry;


	for_each_pci_dev_in_class (class, mask, dev) {

		if (! dev->driver || ! dev->driver->name || strcmp (SERIAL_DRIVER, dev->driver->name) ) 
			continue;

		entry = pci_match_id (ids, dev);
		if (entry && entry->vendor != PCI_ANY_ID) {
			pr_info("ungrab-serial: detaching %x:%x from %s\n", 
				dev->vendor, dev->device, dev->driver->name);

			device_release_driver (& dev->dev);
		}
	}
}

void detach_from_serial(struct pci_device_id *ids) 
{
	detach_from_serial_class (PCI_CLASS_COMMUNICATION_SERIAL << 8, 0xffff00, ids);
	detach_from_serial_class (PCI_CLASS_COMMUNICATION_MODEM << 8, 0xffff00, ids);
	detach_from_serial_class (PCI_CLASS_COMMUNICATION_MULTISERIAL << 8, 0xffff00, ids);
	detach_from_serial_class (PCI_CLASS_COMMUNICATION_OTHER << 8, 0xffff00, ids);
}
EXPORT_SYMBOL(detach_from_serial);


/***** kernel module interface *****/

static int __init cleaner_init (void) 
{
	return 0;
}

static void __exit cleaner_exit(void) 
{}

module_exit (cleaner_exit);
module_init (cleaner_init);

/***** module info *****/

MODULE_LICENSE ("GPL");
MODULE_DESCRIPTION ("Release pci serial device");
